@@ -209,6 +209,9 @@ struct _ClutterTransformInfo
   /* transformation center */
   ClutterPoint pivot;
   gfloat pivot_z;
+
+  CoglMatrix transform;
+  guint transform_set : 1;
 };
 
 const ClutterTransformInfo *    _clutter_actor_get_transform_info_or_defaults   (ClutterActor *self);
@@ -236,9 +239,6 @@ ClutterTransition *             _clutter_actor_create_transition
                                                                                  ...);
 ClutterTransition *             _clutter_actor_get_transition                   (ClutterActor *self,
                                                                                  GParamSpec   *pspec);
-void                            _clutter_actor_update_transition                (ClutterActor *self,
-                                                                                 GParamSpec   *pspec,
-                                                                                 ...);
 
 gboolean      _clutter_actor_foreach_child              (ClutterActor *self,
                                                          ClutterForeachCallback callback,
