@@ -1142,6 +1142,7 @@ clutter_event_copy (const ClutterEvent *event)
       new_real_event->source_device = real_event->source_device;
       new_real_event->delta_x = real_event->delta_x;
       new_real_event->delta_y = real_event->delta_y;
+      new_real_event->is_pointer_emulated = real_event->is_pointer_emulated;
     }
 
   device = clutter_event_get_device (event);
@@ -1626,3 +1627,26 @@ clutter_event_has_control_modifier (const ClutterEvent *event)
 {
   return (clutter_event_get_state (event) & CLUTTER_CONTROL_MASK) != FALSE;
 }
+
+/**
+ * clutter_event_is_pointer_emulated:
+ * @event: a #ClutterEvent
+ *
+ * Checks whether a pointer @event has been generated by the windowing
+ * system. The returned value can be used to distinguish between events
+ * synthesized by the windowing system itself (as opposed by Clutter).
+ *
+ * Return value: %TRUE if the event is pointer emulated
+ *
+ * Since: 1.12
+ */
+gboolean
+clutter_event_is_pointer_emulated (const ClutterEvent *event)
+{
+  g_return_val_if_fail (event != NULL, FALSE);
+
+  if (!is_event_allocated (event))
+    return FALSE;
+
+  return ((ClutterEventPrivate *) event)->is_pointer_emulated;
+}
