@@ -58,16 +58,19 @@
 #include <gobject/gvaluecollector.h>
 
 #include "clutter-color.h"
-#include "clutter-fixed.h"
 #include "clutter-interval.h"
 #include "clutter-private.h"
 #include "clutter-units.h"
 
+#include "deprecated/clutter-fixed.h"
+
 enum
 {
   PROP_0,
 
   PROP_VALUE_TYPE,
+  PROP_INITIAL,
+  PROP_FINAL,
 
   PROP_LAST
 };
@@ -133,6 +136,21 @@ clutter_interval_real_validate (ClutterInterval *interval,
       }
       break;
 
+    case G_TYPE_INT64:
+      {
+        GParamSpecInt64 *pspec_int = G_PARAM_SPEC_INT64 (pspec);
+        gint64 a, b;
+
+        a = b = 0;
+        clutter_interval_get_interval (interval, &a, &b);
+        if ((a >= pspec_int->minimum && a <= pspec_int->maximum) &&
+            (b >= pspec_int->minimum && b <= pspec_int->maximum))
+          return TRUE;
+        else
+          return FALSE;
+      }
+      break;
+
     case G_TYPE_UINT:
       {
         GParamSpecUInt *pspec_uint = G_PARAM_SPEC_UINT (pspec);
@@ -148,6 +166,36 @@ clutter_interval_real_validate (ClutterInterval *interval,
       }
       break;
 
+    case G_TYPE_UINT64:
+      {
+        GParamSpecUInt64 *pspec_int = G_PARAM_SPEC_UINT64 (pspec);
+        guint64 a, b;
+
+        a = b = 0;
+        clutter_interval_get_interval (interval, &a, &b);
+        if ((a >= pspec_int->minimum && a <= pspec_int->maximum) &&
+            (b >= pspec_int->minimum && b <= pspec_int->maximum))
+          return TRUE;
+        else
+          return FALSE;
+      }
+      break;
+
+    case G_TYPE_CHAR:
+      {
+        GParamSpecChar *pspec_char = G_PARAM_SPEC_CHAR (pspec);
+        guchar a, b;
+
+        a = b = 0;
+        clutter_interval_get_interval (interval, &a, &b);
+        if ((a >= pspec_char->minimum && a <= pspec_char->maximum) &&
+            (b >= pspec_char->minimum && b <= pspec_char->maximum))
+          return TRUE;
+        else
+          return FALSE;
+      }
+      break;
+
     case G_TYPE_UCHAR:
       {
         GParamSpecUChar *pspec_uchar = G_PARAM_SPEC_UCHAR (pspec);
@@ -163,6 +211,36 @@ clutter_interval_real_validate (ClutterInterval *interval,
       }
       break;
 
+    case G_TYPE_FLOAT:
+      {
+        GParamSpecFloat *pspec_flt = G_PARAM_SPEC_FLOAT (pspec);
+        float a, b;
+
+        a = b = 0.f;
+        clutter_interval_get_interval (interval, &a, &b);
+        if ((a >= pspec_flt->minimum && a <= pspec_flt->maximum) &&
+            (b >= pspec_flt->minimum && b <= pspec_flt->maximum))
+          return TRUE;
+        else
+          return FALSE;
+      }
+      break;
+
+    case G_TYPE_DOUBLE:
+      {
+        GParamSpecDouble *pspec_flt = G_PARAM_SPEC_DOUBLE (pspec);
+        double a, b;
+
+        a = b = 0;
+        clutter_interval_get_interval (interval, &a, &b);
+        if ((a >= pspec_flt->minimum && a <= pspec_flt->maximum) &&
+            (b >= pspec_flt->minimum && b <= pspec_flt->maximum))
+          return TRUE;
+        else
+          return FALSE;
+      }
+      break;
+
     case G_TYPE_BOOLEAN:
       return TRUE;
 
@@ -215,6 +293,21 @@ clutter_interval_real_compute_value (ClutterInterval *interval,
       }
       break;
 
+    case G_TYPE_CHAR:
+      {
+        gchar ia, ib, res;
+
+        ia = g_value_get_schar (initial);
+        ib = g_value_get_schar (final);
+
+        res = (factor * (ib - (gdouble) ia)) + ia;
+
+        g_value_set_schar (value, res);
+
+        retval = TRUE;
+      }
+      break;
+
     case G_TYPE_UINT:
       {
         guint ia, ib, res;
@@ -292,7 +385,7 @@ clutter_interval_real_compute_value (ClutterInterval *interval,
    * a warning with a hint to what could be done to fix that */
   if (G_UNLIKELY (retval == FALSE))
     {
-      g_warning ("%s: Could not compute progress between two %ss. You can "
+      g_warning ("%s: Could not compute progress between two %s. You can "
                  "register a progress function to instruct ClutterInterval "
                  "how to deal with this GType",
                  G_STRLOC,
@@ -307,8 +400,14 @@ clutter_interval_finalize (GObject *gobject)
 {
   ClutterIntervalPrivate *priv = CLUTTER_INTERVAL (gobject)->priv;
 
-  g_value_unset (&priv->values[0]);
-  g_value_unset (&priv->values[1]);
+  if (G_IS_VALUE (&priv->values[INITIAL]))
+    g_value_unset (&priv->values[INITIAL]);
+
+  if (G_IS_VALUE (&priv->values[FINAL]))
+    g_value_unset (&priv->values[FINAL]);
+
+  if (G_IS_VALUE (&priv->values[RESULT]))
+    g_value_unset (&priv->values[RESULT]);
 
   g_free (priv->values);
 
@@ -329,6 +428,22 @@ clutter_interval_set_property (GObject      *gobject,
       priv->value_type = g_value_get_gtype (value);
       break;
 
+    case PROP_INITIAL:
+      if (g_value_get_boxed (value) != NULL)
+        clutter_interval_set_initial_value (CLUTTER_INTERVAL (gobject),
+                                            g_value_get_boxed (value));
+      else if (G_IS_VALUE (&priv->values[INITIAL]))
+        g_value_unset (&priv->values[INITIAL]);
+      break;
+
+    case PROP_FINAL:
+      if (g_value_get_boxed (value) != NULL)
+        clutter_interval_set_final_value (CLUTTER_INTERVAL (gobject),
+                                          g_value_get_boxed (value));
+      else if (G_IS_VALUE (&priv->values[FINAL]))
+        g_value_unset (&priv->values[FINAL]);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
       break;
@@ -349,6 +464,16 @@ clutter_interval_get_property (GObject    *gobject,
       g_value_set_gtype (value, priv->value_type);
       break;
 
+    case PROP_INITIAL:
+      if (G_IS_VALUE (&priv->values[INITIAL]))
+        g_value_set_boxed (value, &priv->values[INITIAL]);
+      break;
+
+    case PROP_FINAL:
+      if (G_IS_VALUE (&priv->values[FINAL]))
+        g_value_set_boxed (value, &priv->values[FINAL]);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
       break;
@@ -359,7 +484,6 @@ static void
 clutter_interval_class_init (ClutterIntervalClass *klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
-  GParamSpec *pspec;
 
   g_type_class_add_private (klass, sizeof (ClutterIntervalPrivate));
 
@@ -377,13 +501,46 @@ clutter_interval_class_init (ClutterIntervalClass *klass)
    *
    * Since: 1.0
    */
-  pspec = g_param_spec_gtype ("value-type",
-                              P_("Value Type"),
-                              P_("The type of the values in the interval"),
-                              G_TYPE_NONE,
-                              G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
-  obj_props[PROP_VALUE_TYPE] = pspec;
-  g_object_class_install_property (gobject_class, PROP_VALUE_TYPE, pspec);
+  obj_props[PROP_VALUE_TYPE] =
+    g_param_spec_gtype ("value-type",
+                        P_("Value Type"),
+                        P_("The type of the values in the interval"),
+                        G_TYPE_NONE,
+                        G_PARAM_READWRITE |
+                        G_PARAM_CONSTRUCT_ONLY |
+                        G_PARAM_STATIC_STRINGS);
+
+  /**
+   * ClutterInterval:initial:
+   *
+   * The initial value of the interval.
+   *
+   * Since: 1.12
+   */
+  obj_props[PROP_INITIAL] =
+    g_param_spec_boxed ("initial",
+                        P_("Initial Value"),
+                        P_("Initial value of the interval"),
+                        G_TYPE_VALUE,
+                        G_PARAM_READWRITE |
+                        G_PARAM_STATIC_STRINGS);
+
+  /**
+   * ClutterInterval:final:
+   *
+   * The final value of the interval.
+   *
+   * Since: 1.12
+   */
+  obj_props[PROP_FINAL] =
+    g_param_spec_boxed ("final",
+                        P_("Final Value"),
+                        P_("Final value of the interval"),
+                        G_TYPE_VALUE,
+                        G_PARAM_READWRITE |
+                        G_PARAM_STATIC_STRINGS);
+
+  g_object_class_install_properties (gobject_class, PROP_LAST, obj_props);
 }
 
 static void
@@ -403,6 +560,7 @@ clutter_interval_set_value_internal (ClutterInterval *interval,
                                      const GValue    *value)
 {
   ClutterIntervalPrivate *priv = interval->priv;
+  GType value_type;
 
   g_assert (index_ >= INITIAL && index_ <= RESULT);
 
@@ -410,7 +568,39 @@ clutter_interval_set_value_internal (ClutterInterval *interval,
     g_value_unset (&priv->values[index_]);
 
   g_value_init (&priv->values[index_], priv->value_type);
-  g_value_copy (value, &priv->values[index_]);
+
+  value_type = G_VALUE_TYPE (value);
+  if (value_type != priv->value_type ||
+      !g_type_is_a (value_type, priv->value_type))
+    {
+      if (g_value_type_compatible (value_type, priv->value_type))
+        {
+          g_value_copy (value, &priv->values[index_]);
+          return;
+        }
+
+      if (g_value_type_transformable (value_type, priv->value_type))
+        {
+          GValue transform = G_VALUE_INIT;
+
+          g_value_init (&transform, priv->value_type);
+
+          if (g_value_transform (value, &transform))
+            g_value_copy (&transform, &priv->values[index_]);
+          else
+            {
+              g_warning ("%s: Unable to convert a value of type '%s' into "
+                         "the value type '%s' of the interval.",
+                         G_STRLOC,
+                         g_type_name (value_type),
+                         g_type_name (priv->value_type));
+            }
+
+          g_value_unset (&transform);
+        }
+    }
+  else
+    g_value_copy (value, &priv->values[index_]);
 }
 
 static inline void
@@ -584,21 +774,15 @@ clutter_interval_new_with_values (GType         gtype,
                                   const GValue *initial,
                                   const GValue *final)
 {
-  ClutterInterval *retval;
-
   g_return_val_if_fail (gtype != G_TYPE_INVALID, NULL);
   g_return_val_if_fail (initial == NULL || G_VALUE_TYPE (initial) == gtype, NULL);
   g_return_val_if_fail (final == NULL || G_VALUE_TYPE (final) == gtype, NULL);
 
-  retval = g_object_new (CLUTTER_TYPE_INTERVAL, "value-type", gtype, NULL);
-
-  if (initial != NULL)
-    clutter_interval_set_initial_value (retval, initial);
-
-  if (final != NULL)
-    clutter_interval_set_final_value (retval, final);
-
-  return retval;
+  return g_object_new (CLUTTER_TYPE_INTERVAL,
+                       "value-type", gtype,
+                       "initial", initial,
+                       "final", final,
+                       NULL);
 }
 
 /**
@@ -667,15 +851,9 @@ void
 clutter_interval_set_initial_value (ClutterInterval *interval,
                                     const GValue    *value)
 {
-  ClutterIntervalPrivate *priv;
-
   g_return_if_fail (CLUTTER_IS_INTERVAL (interval));
   g_return_if_fail (value != NULL);
 
-  priv = interval->priv;
-
-  g_return_if_fail (G_VALUE_TYPE (value) == priv->value_type);
-
   clutter_interval_set_value_internal (interval, INITIAL, value);
 }
 
@@ -765,15 +943,9 @@ void
 clutter_interval_set_final_value (ClutterInterval *interval,
                                   const GValue    *value)
 {
-  ClutterIntervalPrivate *priv;
-
   g_return_if_fail (CLUTTER_IS_INTERVAL (interval));
   g_return_if_fail (value != NULL);
 
-  priv = interval->priv;
-
-  g_return_if_fail (G_VALUE_TYPE (value) == priv->value_type);
-
   clutter_interval_set_value_internal (interval, FINAL, value);
 }
 
@@ -1011,3 +1183,27 @@ clutter_interval_compute (ClutterInterval *interval,
 
   return NULL;
 }
+
+/**
+ * clutter_interval_is_valid:
+ * @interval: a #ClutterInterval
+ *
+ * Checks if the @interval has a valid initial and final values.
+ *
+ * Return value: %TRUE if the #ClutterInterval has an initial and
+ *   final values, and %FALSE otherwise
+ *
+ * Since: 1.12
+ */
+gboolean
+clutter_interval_is_valid (ClutterInterval *interval)
+{
+  ClutterIntervalPrivate *priv;
+
+  g_return_val_if_fail (CLUTTER_IS_INTERVAL (interval), FALSE);
+
+  priv = interval->priv;
+
+  return G_IS_VALUE (&priv->values[INITIAL]) &&
+         G_IS_VALUE (&priv->values[FINAL]);
+}
