@@ -44,7 +44,7 @@
 #define CLUTTER_FLAVOUR         "deprecated"
 
 /**
- * CLUTTER_COGL
+ * CLUTTER_COGL:
  *
  * Cogl (internal GL abstraction utility library) backend. Can be "gl" or
  * "gles" currently
@@ -75,7 +75,7 @@
  *
  * Set to 1 if Clutter was built without FPU (i.e fixed math), 0 otherwise
  *
- * @Deprecated: 0.6: This macro is no longer defined (identical code is used
+ * Deprecated: 0.6: This macro is no longer defined (identical code is used
  *  regardless the presence of FPU).
  */
 #define CLUTTER_NO_FPU          (0)
@@ -126,7 +126,7 @@
  * Since: 1.10
  */
 #ifndef CLUTTER_VERSION_MIN_REQUIRED
-# define CLUTTER_VERSION_MIN_REQUIRED   (CLUTTER_VERSION_PREV_STABLE)
+# define CLUTTER_VERSION_MIN_REQUIRED   (CLUTTER_VERSION_CUR_STABLE)
 #endif
 
 /**
@@ -248,24 +248,18 @@
 # define CLUTTER_AVAILABLE_IN_1_10
 #endif
 
-/**
- * CLUTTER_CAIRO_FORMAT_ARGB32:
- *
- * The #CoglPixelFormat to be used when uploading image data from
- * and to a Cairo image surface using %CAIRO_FORMAT_ARGB32 and
- * %CAIRO_FORMAT_RGB24 as #cairo_format_t.
- *
- * Since: 1.8
- */
-
-/* Cairo stores the data in native byte order as ARGB but Cogl's pixel
- * formats specify the actual byte order. Therefore we need to use a
- * different format depending on the architecture
- */
-#if G_BYTE_ORDER == G_LITTLE_ENDIAN
-#define CLUTTER_CAIRO_FORMAT_ARGB32     (COGL_PIXEL_FORMAT_BGRA_8888_PRE)
+#if CLUTTER_VERSION_MIN_REQUIRED >= CLUTTER_VERSION_1_12
+# define CLUTTER_DEPRECATED_IN_1_12             CLUTTER_DEPRECATED
+# define CLUTTER_DEPRECATED_IN_1_12_FOR(f)      CLUTTER_DEPRECATED_FOR(f)
 #else
-#define CLUTTER_CAIRO_FORMAT_ARGB32     (COGL_PIXEL_FORMAT_ARGB_8888_PRE)
+# define CLUTTER_DEPRECATED_IN_1_12
+# define CLUTTER_DEPRECATED_IN_1_12_FOR(f)
+#endif
+
+#if CLUTTER_VERSION_MAX_ALLOWED < CLUTTER_VERSION_1_12
+# define CLUTTER_AVAILABLE_IN_1_12              CLUTTER_UNAVAILABLE(1, 12)
+#else
+# define CLUTTER_AVAILABLE_IN_1_12
 #endif
 
 #endif /* __CLUTTER_MACROS_H__ */
