@@ -193,9 +193,6 @@ multi_layout_allocate (ClutterLayoutManager   *manager,
   int n_items, n_items_per_row = 0, item_index;
   ClutterPoint center = CLUTTER_POINT_INIT_ZERO;
   double radius = 0, theta = 0;
-  gboolean use_animations;
-  ClutterAnimationMode easing_mode;
-  guint easing_duration, easing_delay;
 
   n_items = get_visible_children (CLUTTER_ACTOR (container));
   if (n_items == 0)
@@ -224,11 +221,6 @@ multi_layout_allocate (ClutterLayoutManager   *manager,
                     (avail_height - self->cell_height) / 2.0);
     }
 
-  use_animations = clutter_layout_manager_get_easing_state (manager,
-                                                            &easing_mode,
-                                                            &easing_duration,
-                                                            &easing_delay);
-
   clutter_actor_iter_init (&iter, CLUTTER_ACTOR (container));
   while (clutter_actor_iter_next (&iter, &child))
     {
@@ -262,19 +254,8 @@ multi_layout_allocate (ClutterLayoutManager   *manager,
           child_allocation.y2 = child_allocation.y1 + self->cell_height;
         }
 
-      if (use_animations)
-        {
-          clutter_actor_save_easing_state (child);
-          clutter_actor_set_easing_mode (child, easing_mode);
-          clutter_actor_set_easing_duration (child, easing_duration);
-          clutter_actor_set_easing_delay (child, easing_delay);
-        }
-
       clutter_actor_allocate (child, &child_allocation, flags);
 
-      if (use_animations)
-        clutter_actor_restore_easing_state (child);
-
       item_index += 1;
     }
 }
@@ -333,6 +314,28 @@ multi_layout_set_spacing (MultiLayout *self,
 #define PADDING         12.0
 #define BOX_SIZE        (RECT_SIZE * (N_RECTS / N_ROWS) + PADDING * (N_RECTS / N_ROWS - 1))
 
+static gboolean
+on_enter (ClutterActor *rect,
+          ClutterEvent *event)
+{
+  clutter_actor_save_easing_state (rect);
+  clutter_actor_set_scale (rect, 1.2, 1.2);
+  clutter_actor_restore_easing_state (rect);
+
+  return CLUTTER_EVENT_STOP;
+}
+
+static gboolean
+on_leave (ClutterActor *rect,
+          ClutterEvent *event)
+{
+  clutter_actor_save_easing_state (rect);
+  clutter_actor_set_scale (rect, 1.0, 1.0);
+  clutter_actor_restore_easing_state (rect);
+
+  return CLUTTER_EVENT_STOP;
+}
+
 static gboolean
 on_key_press (ClutterActor *stage,
               ClutterEvent *event,
@@ -382,7 +385,6 @@ main (int argc, char *argv[])
 
   manager = multi_layout_new ();
   multi_layout_set_spacing ((MultiLayout *) manager, PADDING);
-  clutter_layout_manager_set_use_animations (manager, TRUE);
 
   margin.top = margin.bottom = margin.left = margin.right = PADDING;
 
@@ -406,8 +408,15 @@ main (int argc, char *argv[])
       color.alpha = 128 + 128 / N_RECTS * i;
 
       clutter_actor_set_size (rect, RECT_SIZE, RECT_SIZE);
+      clutter_actor_set_pivot_point (rect, .5f, .5f);
       clutter_actor_set_background_color (rect, &color);
+      clutter_actor_set_reactive (rect, TRUE);
+      clutter_actor_set_easing_duration (rect, 250);
+      clutter_actor_set_easing_mode (rect, CLUTTER_EASE_OUT_CUBIC);
       clutter_actor_add_child (box, rect);
+
+      g_signal_connect (rect, "enter-event", G_CALLBACK (on_enter), NULL);
+      g_signal_connect (rect, "leave-event", G_CALLBACK (on_leave), NULL);
     }
 
   label = clutter_text_new ();
